@@ -33,6 +33,8 @@ from firewall.config.dbus import *
 from firewall.dbus_utils import dbus_to_python
 import dbus
 from decorator import decorator
+from firewall.functions import b2u
+from firewall.core.rich import Rich_Rule
 
 exception_handler = None
 
@@ -45,26 +47,26 @@ def handle_exceptions(func, *args, **kwargs):
     try:
         return func(*args, **kwargs)
     except dbus.exceptions.DBusException as e:
-        dbus_message = e.get_dbus_message()
+        dbus_message = e.get_dbus_message() # returns unicode
         dbus_name = e.get_dbus_name()
         if not exception_handler:
             raise
         if "NotAuthorizedException" in dbus_name:
-            exception_handler(_("Authorization failed."))
+            exception_handler("NotAuthorizedException")
         else:
             if dbus_message:
                 exception_handler(dbus_message)
             else:
-                exception_handler(str(e))
+                exception_handler(b2u(str(e)))
     except Exception as e:
         if not exception_handler:
             raise
         else:
-            exception_handler(str(e))
+            exception_handler(b2u(str(e)))
 
 # zone config setings
 
-class FirewallClientConfigZoneSettings(object):
+class FirewallClientZoneSettings(object):
     @handle_exceptions
     def __init__(self, settings = None):
         if settings:
@@ -105,7 +107,7 @@ class FirewallClientConfigZoneSettings(object):
 
     @handle_exceptions
     def getServices(self):
-        return self.settings[5]
+        return sorted(self.settings[5])
     @handle_exceptions
     def setServices(self, services):
         self.settings[5] = services
@@ -141,7 +143,7 @@ class FirewallClientConfigZoneSettings(object):
 
     @handle_exceptions
     def getIcmpBlocks(self):
-        return self.settings[7]
+        return sorted(self.settings[7])
     @handle_exceptions
     def setIcmpBlocks(self, icmpblocks):
         self.settings[7] = icmpblocks
@@ -196,7 +198,7 @@ class FirewallClientConfigZoneSettings(object):
 
     @handle_exceptions
     def getInterfaces(self):
-        return self.settings[10]
+        return sorted(self.settings[10])
     @handle_exceptions
     def setInterfaces(self, interfaces):
         self.settings[10] = interfaces
@@ -214,7 +216,7 @@ class FirewallClientConfigZoneSettings(object):
 
     @handle_exceptions
     def getSources(self):
-        return self.settings[11]
+        return sorted(self.settings[11])
     @handle_exceptions
     def setSources(self, sources):
         self.settings[11] = sources
@@ -235,17 +237,21 @@ class FirewallClientConfigZoneSettings(object):
         return self.settings[12]
     @handle_exceptions
     def setRichRules(self, rules):
+        rules = [ str(Rich_Rule(rule_str=r)) for r in rules ]
         self.settings[12] = rules
     @handle_exceptions
     def addRichRule(self, rule):
+        rule = str(Rich_Rule(rule_str=rule))
         if rule not in self.settings[12]:
             self.settings[12].append(rule)
     @handle_exceptions
     def removeRichRule(self, rule):
+        rule = str(Rich_Rule(rule_str=rule))
         if rule in self.settings[12]:
             self.settings[12].remove(rule)
     @handle_exceptions
     def queryRichRule(self, rule):
+        rule = str(Rich_Rule(rule_str=rule))
         return rule in self.settings[12]
 
 
@@ -282,7 +288,7 @@ class FirewallClientConfigZone(object):
     @slip.dbus.polkit.enable_proxy
     @handle_exceptions
     def getSettings(self):
-        return FirewallClientConfigZoneSettings(list(dbus_to_python(\
+        return FirewallClientZoneSettings(list(dbus_to_python(\
                     self.fw_zone.getSettings())))
 
     @slip.dbus.polkit.enable_proxy
@@ -356,7 +362,7 @@ class FirewallClientServiceSettings(object):
 
     @handle_exceptions
     def getModules(self):
-        return self.settings[4]
+        return sorted(self.settings[4])
     @handle_exceptions
     def setModules(self, modules):
         self.settings[4] = modules
@@ -490,9 +496,16 @@ class FirewallClientIcmpTypeSettings(object):
     def removeDestination(self, destination):
         if destination in self.settings[3]:
             self.settings[3].remove(destination)
+        # empty means all
+        elif not self.settings[3]:
+            self.setDestinations(list(set(['ipv4','ipv6']) - \
+                                      set([destination])))
+
     @handle_exceptions
     def queryDestination(self, destination):
-        return destination in self.settings[3]
+        # empty means all
+        return not self.settings[3] or \
+               destination in self.settings[3]
 
 # icmptype config
 
@@ -551,7 +564,7 @@ class FirewallClientConfigIcmpType(object):
 
 # config.policies lockdown whitelist
 
-class FirewallClientConfigPoliciesLockdownWhitelist(object):
+class FirewallClientPoliciesLockdownWhitelist(object):
     @handle_exceptions
     def __init__(self, settings=None):
         if settings:
@@ -561,7 +574,7 @@ class FirewallClientConfigPoliciesLockdownWhitelist(object):
 
     @handle_exceptions
     def getCommands(self):
-        return self.settings[0]
+        return sorted(self.settings[0])
     @handle_exceptions
     def setCommands(self, commands):
         self.settings[0] = commands
@@ -579,7 +592,7 @@ class FirewallClientConfigPoliciesLockdownWhitelist(object):
 
     @handle_exceptions
     def getContexts(self):
-        return self.settings[1]
+        return sorted(self.settings[1])
     @handle_exceptions
     def setContexts(self, contexts):
         self.settings[1] = contexts
@@ -597,7 +610,7 @@ class FirewallClientConfigPoliciesLockdownWhitelist(object):
 
     @handle_exceptions
     def getUsers(self):
-        return self.settings[2]
+        return sorted(self.settings[2])
     @handle_exceptions
     def setUsers(self, users):
         self.settings[2] = users
@@ -615,7 +628,7 @@ class FirewallClientConfigPoliciesLockdownWhitelist(object):
 
     @handle_exceptions
     def getUids(self):
-        return self.settings[3]
+        return sorted(self.settings[3])
     @handle_exceptions
     def setUids(self, uids):
         self.settings[3] = uids
@@ -645,7 +658,7 @@ class FirewallClientConfigPolicies(object):
     @slip.dbus.polkit.enable_proxy
     @handle_exceptions
     def getLockdownWhitelist(self):
-        return FirewallClientConfigPoliciesLockdownWhitelist( \
+        return FirewallClientPoliciesLockdownWhitelist( \
             list(dbus_to_python(self.fw_policies.getLockdownWhitelist())))
 
     @slip.dbus.polkit.enable_proxy
@@ -653,6 +666,114 @@ class FirewallClientConfigPolicies(object):
     def setLockdownWhitelist(self, settings):
         self.fw_policies.setLockdownWhitelist(tuple(settings.settings))
 
+# config.direct
+
+class FirewallClientDirect(object):
+    @handle_exceptions
+    def __init__(self, settings=None):
+        if settings:
+            self.settings = settings
+        else:
+            self.settings = [ [], [], [], ]
+
+    @handle_exceptions
+    def getAllChains(self):
+        return self.settings[0]
+    @handle_exceptions
+    def getChains(self, ipv, table):
+        return [ entry[2] for entry in self.settings[0] \
+                 if entry[0] == ipv and entry[1] == table ]
+    @handle_exceptions
+    def setAllChains(self, chains):
+        self.settings[0] = chains
+    @handle_exceptions
+    def addChain(self, ipv, table, chain):
+        idx = (ipv, table, chain)
+        if idx not in self.settings[0]:
+            self.settings[0].append(idx)
+    @handle_exceptions
+    def removeChain(self, ipv, table, chain):
+        idx = (ipv, table, chain)
+        if idx in self.settings[0]:
+            self.settings[0].remove(idx)
+    @handle_exceptions
+    def queryChain(self, ipv, table, chain):
+        idx = (ipv, table, chain)
+        return idx in self.settings[0]
+
+    @handle_exceptions
+    def getAllRules(self):
+        return self.settings[1]
+    @handle_exceptions
+    def getRules(self, ipv, table, chain):
+        return [ entry[3:] for entry in self.settings[1] \
+                 if entry[0] == ipv and entry[1] == table \
+                 and entry[2] == chain ]
+    @handle_exceptions
+    def setAllRules(self, rules):
+        self.settings[1] = rules
+    @handle_exceptions
+    def addRule(self, ipv, table, chain, priority, args):
+        idx = (ipv, table, chain, priority, args)
+        if idx not in self.settings[1]:
+            self.settings[1].append(idx)
+    @handle_exceptions
+    def removeRule(self, ipv, table, chain, priority, args):
+        idx = (ipv, table, chain, priority, args)
+        if idx in self.settings[1]:
+            self.settings[1].remove(idx)
+    @handle_exceptions
+    def queryRule(self, ipv, table, chain, priority, args):
+        idx = (ipv, table, chain, priority, args)
+        return idx in self.settings[1]
+
+    @handle_exceptions
+    def getAllPassthroughs(self):
+        return self.settings[2]
+    @handle_exceptions
+    def getPassthroughs(self, ipv):
+        return [ entry[1] for entry in self.settings[2] \
+                 if entry[0] == ipv ]
+    @handle_exceptions
+    def setAllPassthroughs(self, passthroughs):
+        self.settings[2] = passthroughs
+    @handle_exceptions
+    def addPassthrough(self, ipv, args):
+        idx = (ipv, args)
+        if idx not in self.settings[2]:
+            self.settings[2].append(idx)
+    @handle_exceptions
+    def removePassthrough(self, ipv, args):
+        idx = (ipv, args)
+        if idx in self.settings[2]:
+            self.settings[2].remove(idx)
+    @handle_exceptions
+    def queryPassthrough(self, ipv, args):
+        idx = (ipv, args)
+        return idx in self.settings[2]
+
+# config.direct
+
+class FirewallClientConfigDirect(object):
+    @handle_exceptions
+    def __init__(self, bus):
+        self.bus = bus
+        self.dbus_obj = self.bus.get_object(DBUS_INTERFACE,
+                                            DBUS_PATH_CONFIG)
+        self.fw_direct = dbus.Interface( \
+            self.dbus_obj, dbus_interface=DBUS_INTERFACE_CONFIG_DIRECT)
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def getSettings(self):
+        return FirewallClientDirect( \
+            list(dbus_to_python(self.fw_direct.getSettings())))
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def update(self, settings):
+        self.fw_direct.update(tuple(settings.settings))
+
 # config
 
 class FirewallClientConfig(object):
@@ -666,6 +787,7 @@ class FirewallClientConfig(object):
         self.fw_properties = dbus.Interface(
             self.dbus_obj, dbus_interface='org.freedesktop.DBus.Properties')
         self._policies = FirewallClientConfigPolicies(self.bus)
+        self._direct = FirewallClientConfigDirect(self.bus)
 
     # properties
 
@@ -771,6 +893,12 @@ class FirewallClientConfig(object):
     @handle_exceptions
     def policies(self):
         return self._policies
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def direct(self):
+        return self._direct
+
 #
 
 class FirewallClient(object):
@@ -799,6 +927,7 @@ class FirewallClient(object):
                            DBUS_INTERFACE_CONFIG,
                            DBUS_INTERFACE_CONFIG_ZONE,
                            DBUS_INTERFACE_CONFIG_SERVICE,
+                           DBUS_INTERFACE_CONFIG_DIRECT,
                            DBUS_INTERFACE_CONFIG_ICMPTYPE,
                            DBUS_INTERFACE_CONFIG_POLICIES ]:
             self.bus.add_signal_receiver(self._signal_receiver,
@@ -839,13 +968,12 @@ class FirewallClient(object):
             "source-removed": "SourceRemoved",
             "zone-of-source-changed": "ZoneOfSourceChanged",
             # direct callbacks
-# TODO
-            "direct-chain-added": "ChainAdded",
-            "direct-chain-removed": "ChainRemoved",
-            "direct-rule-added": "RuleAdded",
-            "direct-rule-removed": "RuleRemoved",
+            "direct:chain-added": "ChainAdded",
+            "direct:chain-removed": "ChainRemoved",
+            "direct:rule-added": "RuleAdded",
+            "direct:rule-removed": "RuleRemoved",
+            "config:direct:updated": "config:direct:Updated",
             # policy callbacks
-# TODO
             "lockdown-enabled": "LockdownEnabled",
             "lockdown-disabled": "LockdownDisabled",
             "lockdown-whitelist-command-added": "LockdownWhitelistCommandAdded",
@@ -985,6 +1113,8 @@ class FirewallClient(object):
             signal = "config:" + signal
         elif interface == DBUS_INTERFACE_CONFIG_POLICIES:
             signal = "config:policies:" + signal
+        elif interface == DBUS_INTERFACE_CONFIG_DIRECT:
+            signal = "config:direct:" + signal
 
         for callback in self._callbacks:
             if self._callbacks[callback] == signal and \
@@ -1343,6 +1473,11 @@ class FirewallClient(object):
     def getChains(self, ipv, table):
         return dbus_to_python(self.fw_direct.getChains(ipv, table))
 
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def getAllChains(self):
+        return dbus_to_python(self.fw_direct.getAllChains())
+
     # direct rule
 
     @slip.dbus.polkit.enable_proxy
@@ -1365,6 +1500,11 @@ class FirewallClient(object):
     def getRules(self, ipv, table, chain):
         return dbus_to_python(self.fw_direct.getRules(ipv, table, chain))
 
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def getAllRules(self):
+        return dbus_to_python(self.fw_direct.getAllRules())
+
     # direct passthrough
 
     @slip.dbus.polkit.enable_proxy
