@@ -58,6 +58,18 @@ class ebtables:
     def delete_rule(self, rule):
         self.__run([ "-D" ] + rule)
 
+    def available_tables(self, table=None):
+        ret = []
+        tables = [ table ] if table else CHAINS.keys()
+        for table in tables:
+            try:
+                self.__run(["-t", table, "-L"])
+                ret.append(table)
+            except ValueError:
+                log.warning("ebtables table '%s' does not exist." % table)
+
+        return ret
+
     def used_tables(self):
         return list(CHAINS.keys())
 
@@ -82,3 +94,5 @@ class ebtables:
         for table in tables:
             for chain in CHAINS[table]:
                 self.__run([ "-t", table, "-P", chain, policy ])
+
+ebtables_available_tables = ebtables().available_tables()
