@@ -77,33 +77,26 @@ class Firewall:
         self._min_mark = FALLBACK_MINIMAL_MARK # will be overloaded by firewalld.conf
         self.cleanup_on_exit = True
 
-    def start(self):
+    def _check_tables(self):
         # check if iptables, ip6tables and ebtables are usable, else disable
-
-        try:
-            self.rule("ipv4", [ "-L" ])
-        except:
+        if not "filter" in ipXtables.ip4tables_available_tables:
             log.warning("iptables not usable, disabling IPv4 firewall.")
             self.ip4tables_enabled = False
-        try:
-            self.rule("ipv6", [ "-L" ])
-        except:
+
+        if not "filter" in ipXtables.ip6tables_available_tables:
             log.warning("ip6tables not usable, disabling IPv6 firewall.")
             self.ip6tables_enabled = False
-        try:
-            self.rule("eb", [ "-L" ])
-        except:
+
+        if not "filter" in ebtables.ebtables_available_tables:
             log.error("ebtables not usable, disabling ethernet bridge firewall.")
             self.ebtables_enabled = False
 
         if not self.ip4tables_enabled and not self.ip6tables_enabled:
-            log.fatal("No IPv4 and IPv6 firewall.")            
+            log.fatal("No IPv4 and IPv6 firewall.")
             sys.exit(1)
 
+    def _start(self):
         # initialize firewall
-        self._flush()
-        self._set_policy("ACCEPT")
-
         default_zone = FALLBACK_ZONE
 
         # load firewalld config
@@ -195,12 +188,15 @@ class Firewall:
             log.debug1("Failed to load direct rules file '%s': %s",
                       FIREWALLD_DIRECT, msg)
         else:
-            #obj.output()
             self.direct.set_config((obj.get_all_chains(), obj.get_all_rules()))
             for ipv, args in obj.get_all_passthroughs().items():
                 for arg in args:
-                    self.direct.passthrough(ipv, arg)
+                    try:
+                        self.direct.passthrough(ipv, arg)
+                    except FirewallError as error:
+                        log.warning(str(error))
             # TODO: copy obj into config interface
+        self.config.set_direct(copy.deepcopy(obj))
 
         # check if default_zone is a valid zone
         if default_zone not in self.zone.get_zones():
@@ -222,6 +218,12 @@ class Firewall:
 
         self._state = "RUNNING"
 
+    def start(self):
+        self._check_tables()
+        self._flush()
+        self._set_policy("ACCEPT")
+        self._start()
+
     def _loader(self, path, reader_type, combine=False):
         # combine: several zone files are getting combined into one obj
         if not os.path.isdir(path):
@@ -361,6 +363,16 @@ class Firewall:
             if insert and not enable and isinstance(rule[1], int):
                 rule.pop(1)
 
+            table = None
+            for t in ipXtables.CHAINS.keys():
+                if t in rule:
+                    table = t
+            if table and not self.is_table_available(ipv, table):
+                if ((ipv == "ipv4" and self.ip4tables_enabled) or
+                    (ipv == "ipv6" and self.ip6tables_enabled)):
+                    log.error("Unable to add %s into %s %s" % (rule, ipv, table))
+                continue
+
             # run
             try:
                 self.rule(ipv, [ append_delete[enable], ] + rule)
@@ -387,6 +399,12 @@ class Firewall:
             if insert and not enable and isinstance(rule[1], int):
                 rule.pop(1)
 
+            if not self.is_table_available(ipv, table):
+                if ((ipv == "ipv4" and self.ip4tables_enabled) or
+                    (ipv == "ipv6" and self.ip6tables_enabled)):
+                    log.error("Unable to add %s into %s %s" % (rule, ipv, table))
+                continue
+
             # run
             try:
                 self.rule(ipv, [ "-t", table,
@@ -433,8 +451,12 @@ class Firewall:
                         del self._module_refcount[module]
         return None
 
-    # apply default rules
+    def is_table_available(self, ipv, table):
+        return ((ipv == "ipv4" and table in ipXtables.ip4tables_available_tables) or
+                (ipv == "ipv6" and table in ipXtables.ip6tables_available_tables) or
+                (ipv == "eb" and table in ebtables.ebtables_available_tables))
 
+    # apply default rules
     def __apply_default_rules(self, ipv):
         if ipv in [ "ipv4", "ipv6" ]:
             default_rules = ipXtables.DEFAULT_RULES
@@ -442,6 +464,8 @@ class Firewall:
             default_rules = ebtables.DEFAULT_RULES
 
         for table in default_rules:
+            if not self.is_table_available(ipv, table):
+                continue
             prefix = [ "-t", table ]
             for rule in default_rules[table]:
                 _rule = prefix + rule.split()
@@ -461,12 +485,16 @@ class Firewall:
     # flush and policy
 
     def _flush(self):
-        self._ip4tables.flush()
-        self._ip6tables.flush()
+        if self.ip4tables_enabled:
+            self._ip4tables.flush()
+        if self.ip6tables_enabled:
+            self._ip6tables.flush()
 
     def _set_policy(self, policy, which="used"):
-        self._ip4tables.set_policy(policy, which)
-        self._ip6tables.set_policy(policy, which)
+        if self.ip4tables_enabled:
+            self._ip4tables.set_policy(policy, which)
+        if self.ip6tables_enabled:
+            self._ip6tables.set_policy(policy, which)
 
     # rule function used in handle_ functions
 
@@ -509,6 +537,8 @@ class Firewall:
         else:
             raise FirewallError(INVALID_IPV, ipv)
 
+        return ""
+
     # check functions
 
     def check_panic(self):
@@ -581,17 +611,15 @@ class Firewall:
         _direct_config = self.direct.get_config()
         _old_dz = self.get_default_zone()
 
+        # stop
+        self._set_policy("DROP")
+        self._flush()
         if stop:
-            self.stop()
-        else:
-            self.cleanup()
-            self._flush()
-            self._set_policy("ACCEPT")
-        self.start()
+            self._modules.unload_firewall_modules()
+        self.cleanup()
 
         # start
-        if _panic:
-            self.enable_panic_mode()
+        self._start()
 
         # handle interfaces in the default zone and move them to the new 
         # default zone if it changed
@@ -625,6 +653,13 @@ class Firewall:
         # restore direct config
         self.direct.set_config(_direct_config)
 
+        # enable panic mode again if it has been enabled before or set policy 
+        # to ACCEPT
+        if _panic:
+            self.enable_panic_mode()
+        else:
+            self._set_policy("ACCEPT")
+
     # STATE
 
     def get_state(self):
