@@ -44,6 +44,7 @@ class FirewallConfig:
         self._default_zones = { }
         self._firewalld_conf = None
         self._policies = None
+        self._direct = None
 
     def cleanup(self):
         for x in self._default_icmptypes.keys():
@@ -87,11 +88,31 @@ class FirewallConfig:
     def get_policies(self):
         return self._policies
 
+    def update_lockdown_whitelist(self):
+        if not os.path.exists(LOCKDOWN_WHITELIST):
+            self._policies.lockdown_whitelist.clear()
+        else:
+            self._policies.lockdown_whitelist.read()
+
+    # direct
+
+    def set_direct(self, direct):
+        self._direct = direct
+
+    def get_direct(self):
+        return self._direct
+
+    def update_direct(self):
+        if not os.path.exists(FIREWALLD_DIRECT):
+            self._direct.clear()
+        else:
+            self._direct.read()
+
     # icmptypes
 
     def get_icmptypes(self):
-        return list(set(list(self._icmptypes.keys()) + \
-                        list(self._default_icmptypes.keys())))
+        return sorted(set(self._icmptypes.keys() + \
+                          self._default_icmptypes.keys()))
 
     def add_icmptype(self, obj):
         if obj.default:
@@ -252,8 +273,8 @@ class FirewallConfig:
     # services
 
     def get_services(self):
-        return list(set(list(self._services.keys()) + \
-                        list(self._default_services.keys())))
+        return sorted(set(self._services.keys() + \
+                          self._default_services.keys()))
 
     def add_service(self, obj):
         if obj.default:
@@ -414,8 +435,8 @@ class FirewallConfig:
     # zones
 
     def get_zones(self):
-        return list(set(self._zones.keys() + \
-                        self._default_zones.keys()))
+        return sorted(set(self._zones.keys() + \
+                          self._default_zones.keys()))
 
     def add_zone(self, obj):
         if obj.default:
