@@ -26,6 +26,49 @@ from firewall.functions import portStr, checkIPnMask, checkIP6nMask, \
     checkProtocol, enable_ip_forwarding, check_single_address
 from firewall.core.rich import *
 from firewall.errors import *
+from ipXtables import ip4tables_available_tables, ip6tables_available_tables
+
+mangle = []
+if "mangle" in ip4tables_available_tables:
+    mangle.append("ipv4")
+if "mangle" in ip6tables_available_tables:
+    mangle.append("ipv6")
+
+nat = []
+if "nat" in ip4tables_available_tables:
+    nat.append("ipv4")
+else:
+    if "ipv4" in mangle:
+        mangle.remove("ipv4")
+if "nat" in ip6tables_available_tables:
+    nat.append("ipv6")
+else:
+    if "ipv6" in mangle:
+        mangle.remove("ipv6")
+
+ZONE_CHAINS = {
+    "filter": {
+        "INPUT": [ "ipv4", "ipv6" ],
+        "FORWARD_IN": [ "ipv4", "ipv6" ],
+        "FORWARD_OUT": [ "ipv4", "ipv6" ],
+        },
+    "nat": {
+        "PREROUTING": nat,
+        "POSTROUTING": nat,
+        },
+    "mangle": {
+        "PREROUTING": mangle,
+        },
+}
+
+INTERFACE_ZONE_OPTS = {
+    "PREROUTING": "-i",
+    "POSTROUTING": "-o",
+    "INPUT": "-i",
+    "FORWARD_IN": "-i",
+    "FORWARD_OUT": "-o",
+    "OUTPUT": "-o",
+}
 
 class FirewallZone:
     def __init__(self, fw):
@@ -42,7 +85,7 @@ class FirewallZone:
     # zones
 
     def get_zones(self):
-        return list(self._zones.keys())
+        return sorted(self._zones.keys())
 
     def get_zone_of_interface(self, interface):
         interface_id = self.__interface_id(interface)
@@ -135,8 +178,11 @@ class FirewallZone:
 
         # TODO: simplify for one zone only
         for _zone in zones:
-
-            ipvs = [ "ipv4", "ipv6" ]
+            ipvs = []
+            if self._fw.is_table_available("ipv4", table):
+                ipvs.append("ipv4")
+            if self._fw.is_table_available("ipv6", table):
+                ipvs.append("ipv6")
 
             for ipv in ipvs:
                 chains.append((ipv, [ _zone, "-t", table ]))
@@ -420,8 +466,8 @@ class FirewallZone:
     def query_interface(self, zone, interface):
         return self.__interface_id(interface) in self.get_settings(zone)["interfaces"]
 
-    def get_interfaces(self, zone):
-        return list(self.get_settings(zone)["interfaces"].keys())
+    def list_interfaces(self, zone):
+        return sorted(self.get_settings(zone)["interfaces"].keys())
 
     # SOURCES
 
@@ -538,7 +584,7 @@ class FirewallZone:
     def query_source(self, zone, source):
         return self.__source_id(source) in self.get_settings(zone)["sources"]
 
-    def get_sources(self, zone):
+    def list_sources(self, zone):
         return [ k[1] for k in self.get_settings(zone)["sources"].keys() ]
 
     # RICH LANGUAGE
@@ -921,7 +967,7 @@ class FirewallZone:
     def query_rule(self, zone, rule):
         return self.__rule_id(rule) in self.get_settings(zone)["rules"]
 
-    def get_rules(self, zone):
+    def list_rules(self, zone):
         return list(self.get_settings(zone)["rules"].keys())
 
     # SERVICES
@@ -1024,8 +1070,8 @@ class FirewallZone:
     def query_service(self, zone, service):
         return (self.__service_id(service) in self.get_settings(zone)["services"])
 
-    def get_services(self, zone):
-        return list(self.get_settings(zone)["services"].keys())
+    def list_services(self, zone):
+        return sorted(self.get_settings(zone)["services"].keys())
 
     # PORTS
 
@@ -1097,7 +1143,7 @@ class FirewallZone:
     def query_port(self, zone, port, protocol):
         return self.__port_id(port, protocol) in self.get_settings(zone)["ports"]
 
-    def get_ports(self, zone):
+    def list_ports(self, zone):
         return list(self.get_settings(zone)["ports"].keys())
 
     # PROTOCOLS
@@ -1297,7 +1343,7 @@ class FirewallZone:
         forward_id = self.__forward_port_id(port, protocol, toport, toaddr)
         return forward_id in self.get_settings(zone)["forward_ports"]
 
-    def get_forward_ports(self, zone):
+    def list_forward_ports(self, zone):
         return list(self.get_settings(zone)["forward_ports"].keys())
 
     # ICMP BLOCK
@@ -1385,5 +1431,5 @@ class FirewallZone:
     def query_icmp_block(self, zone, icmp):
         return self.__icmp_block_id(icmp) in self.get_settings(zone)["icmp_blocks"]
 
-    def get_icmp_blocks(self, zone):
-        return list(self.get_settings(zone)["icmp_blocks"].keys())
+    def list_icmp_blocks(self, zone):
+        return sorted(self.get_settings(zone)["icmp_blocks"].keys())
