@@ -84,7 +84,6 @@ class icmptype_ContentHandler(IO_Object_ContentHandler):
                     self.item.destination.append(str(x))
 
 def icmptype_reader(filename, path):
-    name = "%s/%s" % (path, filename)
     icmptype = IcmpType()
     if not filename.endswith(".xml"):
         raise FirewallError(INVALID_NAME, filename)
@@ -96,7 +95,9 @@ def icmptype_reader(filename, path):
     handler = icmptype_ContentHandler(icmptype)
     parser = sax.make_parser()
     parser.setContentHandler(handler)
-    parser.parse(name)
+    name = "%s/%s" % (path, filename)
+    with open(name, "r") as f:
+        parser.parse(f)
     return icmptype
 
 def icmptype_writer(icmptype, path=None):
@@ -126,21 +127,21 @@ def icmptype_writer(icmptype, path=None):
 
     # short
     if icmptype.short and icmptype.short != "":
-        if isinstance(icmptype.short, bytes):
-            icmptype.short = icmptype.short.decode('utf-8')
         handler.ignorableWhitespace("  ")
         handler.startElement("short", { })
-        handler.characters(icmptype.short)
+        handler.characters(icmptype.short.decode('utf-8')
+                           if isinstance(icmptype.short, bytes)
+                           else icmptype.short)
         handler.endElement("short")
         handler.ignorableWhitespace("\n")
 
     # description
     if icmptype.description and icmptype.description != "":
-        if isinstance(icmptype.description, bytes):
-            icmptype.description = icmptype.description.decode('utf-8')
         handler.ignorableWhitespace("  ")
         handler.startElement("description", { })
-        handler.characters(icmptype.description)
+        handler.characters(icmptype.description.decode('utf-8')
+                           if isinstance(icmptype.description, bytes)
+                           else icmptype.description)
         handler.endElement("description")
         handler.ignorableWhitespace("\n")
 
