@@ -112,7 +112,6 @@ class service_ContentHandler(IO_Object_ContentHandler):
             self.item.modules.append(str(attrs["name"]))
 
 def service_reader(filename, path):
-    name = "%s/%s" % (path, filename)
     service = Service()
     if not filename.endswith(".xml"):
         raise FirewallError(INVALID_NAME, filename)
@@ -124,7 +123,9 @@ def service_reader(filename, path):
     handler = service_ContentHandler(service)
     parser = sax.make_parser()
     parser.setContentHandler(handler)
-    parser.parse(name)
+    name = "%s/%s" % (path, filename)
+    with open(name, "r") as f:
+        parser.parse(f)
     return service
 
 def service_writer(service, path=None):
@@ -154,21 +155,21 @@ def service_writer(service, path=None):
 
     # short
     if service.short and service.short != "":
-        if isinstance(service.short, bytes):
-            service.short = service.short.decode('utf-8')
         handler.ignorableWhitespace("  ")
         handler.startElement("short", { })
-        handler.characters(service.short)
+        handler.characters(service.short.decode('utf-8')
+                           if isinstance(service.short, bytes)
+                           else service.short)
         handler.endElement("short")
         handler.ignorableWhitespace("\n")
 
     # description
     if service.description and service.description != "":
-        if isinstance(service.description, bytes):
-            service.description = service.description.decode('utf-8')
         handler.ignorableWhitespace("  ")
         handler.startElement("description", { })
-        handler.characters(service.description)
+        handler.characters(service.description.decode('utf-8')
+                           if isinstance(service.description, bytes)
+                           else service.description)
         handler.endElement("description")
         handler.ignorableWhitespace("\n")
 
