@@ -374,12 +374,13 @@ class zone_ContentHandler(IO_Object_ContentHandler):
             level = None
             if "level" in attrs:
                 level = str(attrs["level"])
-                if level not in [ "emerg", "alert", "crit", "err",
-                                  "warn", "notice", "info", "debug" ]:
+                if level not in [ "emerg", "alert", "crit", "error",
+                                  "warning", "notice", "info", "debug" ]:
                     log.error('Invalid rule: Invalid log level')
                     self._rule_error = True
                     return
-            self._rule.log = Rich_Log(str(attrs["prefix"]), level)
+            prefix = str(attrs["prefix"]) if "prefix" in attrs else None
+            self._rule.log = Rich_Log(prefix, level)
             self._limit_ok = self._rule.log
 
         elif name == "audit":
@@ -439,7 +440,6 @@ class zone_ContentHandler(IO_Object_ContentHandler):
             self._limit_ok = None
 
 def zone_reader(filename, path):
-    name = "%s/%s" % (path, filename)
     zone = Zone()
     if not filename.endswith(".xml"):
         raise FirewallError(INVALID_NAME, filename)
@@ -453,7 +453,9 @@ def zone_reader(filename, path):
     handler = zone_ContentHandler(zone)
     parser = sax.make_parser()
     parser.setContentHandler(handler)
-    parser.parse(name)
+    name = "%s/%s" % (path, filename)
+    with open(name, "r") as f:
+        parser.parse(f)
     return zone
 
 def zone_writer(zone, path=None):
@@ -485,21 +487,21 @@ def zone_writer(zone, path=None):
 
     # short
     if zone.short and zone.short != "":
-        if isinstance(zone.short, bytes):
-            zone.short = zone.short.decode('utf-8')
         handler.ignorableWhitespace("  ")
         handler.startElement("short", { })
-        handler.characters(zone.short)
+        handler.characters(zone.short.decode('utf-8')
+                           if isinstance(zone.short, bytes)
+                           else zone.short)
         handler.endElement("short")
         handler.ignorableWhitespace("\n")
 
     # description
     if zone.description and zone.description != "":
-        if isinstance(zone.description, bytes):
-            zone.description = zone.description.decode('utf-8')
         handler.ignorableWhitespace("  ")
         handler.startElement("description", { })
-        handler.characters(zone.description)
+        handler.characters(zone.description.decode('utf-8')
+                           if isinstance(zone.description, bytes)
+                           else zone.description)
         handler.endElement("description")
         handler.ignorableWhitespace("\n")
 
