@@ -54,6 +54,16 @@ ICMP = {
 
 DEFAULT_RULES = { }
 
+DEFAULT_RULES["security"] = [ ]
+for chain in CHAINS["security"]:
+    DEFAULT_RULES["security"].append("-N %s_direct" % chain)
+    DEFAULT_RULES["security"].append("-I %s 1 -j %s_direct" % (chain, chain))
+
+DEFAULT_RULES["raw"] = [ ]
+for chain in CHAINS["raw"]:
+    DEFAULT_RULES["raw"].append("-N %s_direct" % chain)
+    DEFAULT_RULES["raw"].append("-I %s 1 -j %s_direct" % (chain, chain))
+
 DEFAULT_RULES["mangle"] = [ ]
 for chain in CHAINS["mangle"]:
     DEFAULT_RULES["mangle"].append("-N %s_direct" % chain)
@@ -135,17 +145,28 @@ class ip4tables:
     def delete_rule(self, rule):
         self.__run([ "-D" ] + rule)
 
+    def available_tables(self, table=None):
+        ret = []
+        tables = [ table ] if table else CHAINS.keys()
+        for table in tables:
+            try:
+                self.__run(["-t", table, "-L"])
+                ret.append(table)
+            except ValueError:
+                log.error("%s table '%s' does not exist (or not enough permission to check)." % (self.ipv, table))
+
+        return ret
+
     def used_tables(self):
         tables = [ ]
         filename = PROC_IPxTABLE_NAMES[self.ipv]
 
         if os.path.exists(filename):
-            f = open(filename, "r")
-            for line in f.readlines():
-                if not line:
-                    break
-                tables.append(line.strip())
-            f.close()
+            with open(filename, "r") as f:
+                for line in f.readlines():
+                    if not line:
+                        break
+                    tables.append(line.strip())
 
         return tables
 
@@ -174,58 +195,59 @@ class ip4tables:
 class ip6tables(ip4tables):
     ipv = "ipv6"
 
-class ipXtables:
-    def __init__(self, ipv4=True, ipv6=True):
-        self.ipv4 = ipv4
-        self.ipv6 = ipv6
-        self.ip4tables = self.ip6tables = None
-        if ipv4:
-            self.ip4tables = ip4tables()
-        if ipv6:
-            self.ip6tables = ip6tables()
-        if not ipv4 and not ipv6:
-            raise ValueError("ipv4 and ipv6 disabled.")
+ip4tables_available_tables = ip4tables().available_tables()
+ip6tables_available_tables = ip6tables().available_tables()
 
-    def append_rule(self, rule):
-        if self.ip4tables:
-            self.ip4tables.append_rule(rule)
-        try:
-            if self.ip6tables:
-                self.ip6tables.append_rule(rule)
-        except Exception as msg:
-            if self.ip4tables:
-                self.ip4tables.delete_rule(rule)
-            raise
+#class ipXtables:
+#    def __init__(self, ipv4=True, ipv6=True):
+#        self.ip4tables = self.ip6tables = None
+#        if ipv4:
+#            self.ip4tables = ip4tables()
+#        if ipv6:
+#            self.ip6tables = ip6tables()
+#        if not ipv4 and not ipv6:
+#            raise ValueError("ipv4 and ipv6 disabled.")
 
-    def delete_rule(self, rule):
-        if self.ip4tables:
-            self.ip4tables.delete_rule(rule)
-        try:
-            if self.ip6tables:
-                self.ip6tables.delete_rule(rule)
-        except Exception as msg:
-            if self.ip4tables:
-                self.ip4tables.append_rule(rule)
-            raise
+#    def append_rule(self, rule):
+#        if self.ip4tables:
+#            self.ip4tables.append_rule(rule)
+#        try:
+#            if self.ip6tables:
+#                self.ip6tables.append_rule(rule)
+#        except Exception:
+#            if self.ip4tables:
+#                self.ip4tables.delete_rule(rule)
+#            raise
 
-    def used_tables(self):
-        tables = [ ]
-        if ip4tables:
-            tables += self.ip4tables.used_tables()
-        if ip6tables:
-            tables += self.ip6tables.used_tables()
-        return tables
+#    def delete_rule(self, rule):
+#        if self.ip4tables:
+#            self.ip4tables.delete_rule(rule)
+#        try:
+#            if self.ip6tables:
+#                self.ip6tables.delete_rule(rule)
+#        except Exception:
+#            if self.ip4tables:
+#                self.ip4tables.append_rule(rule)
+#            raise
 
-    def flush(self):
-        if ip4tables:
-            self.ip4tables.flush()
-        if ip6tables:
-            self.ip6tables.flush()
+#    def used_tables(self):
+#        tables = [ ]
+#        if ip4tables:
+#            tables += self.ip4tables.used_tables()
+#        if ip6tables:
+#            tables += self.ip6tables.used_tables()
+#        return tables
+
+#    def flush(self):
+#        if ip4tables:
+#            self.ip4tables.flush()
+#        if ip6tables:
+#            self.ip6tables.flush()
         # TODO: in case off error state is inconsistent
 
-    def set_policy(self, policy, which="used"):
-        if ip4tables:
-            self.ip4tables.set_policy(policy, which)
-        if ip6tables:
-            self.ip6tables.set_policy(policy, which)
+#    def set_policy(self, policy, which="used"):
+#        if ip4tables:
+#            self.ip4tables.set_policy(policy, which)
+#        if ip6tables:
+#            self.ip6tables.set_policy(policy, which)
         # TODO: in case off error state is inconsistent
