@@ -78,6 +78,11 @@ class Rich_ForwardPort(object):
         self.protocol = protocol
         self.to_port = to_port
         self.to_address = to_address
+        # do not use None for to_port and to_address, replace it by ""
+        if self.to_port == None:
+            self.to_port = ""
+        if self.to_address == None:
+            self.to_address = ""
 
     def __str__(self):
         return 'forward-port port="%s" protocol="%s"%s%s' % \
@@ -130,6 +135,11 @@ class Rich_Drop(Rich_Accept):
 class Rich_Limit(object):
     def __init__(self, value):
         self.value = value
+        if "/" in self.value:
+            splits = self.value.split("/")
+            if len(splits) == 2 and \
+               splits[1] in [ "second", "minute", "hour", "day" ]:
+                self.value = "%s/%s" % (splits[0], splits[1][:1])
 
     def check(self):
         splits = None
@@ -183,58 +193,18 @@ class Rich_Rule(object):
             self._import_from_string(rule_str)
 
     def _lexer(self, rule_str):
-        """Lexical analysis
-           rule_str.split() would be easier, but we need to take spaces
-           in attribute values into consideration
-        """
+        """ Lexical analysis """
         tokens = []
-        _dict = {}
-        _str = ''
-        state = 'element'
-
-        for c in rule_str:
-            if state == 'element':
-                if c == ' ':
-                    tokens.append({'element':_str})
-                    _str = ''
-                elif c == '=':
-                    _dict['attr_name'] = _str
-                    _str = ''
-                    state = 'attribute_value'
-                else:
-                    _str = _str + c
-            elif state == 'attribute_value':
-                if c == ' ':
-                    _dict['attr_value'] = _str
-                    tokens.append(_dict)
-                    _str = ''; _dict = {}
-                    state = 'element'
-                elif c == '"':
-                    state = 'attribute_value_quoted'
-                else:
-                    _str = _str + c
-            elif state == 'attribute_value_quoted':
-                if c == '"':
-                    state = 'attribute_value'
-                else:
-                    _str = _str + c
-
-        if state == 'element' and _str:
-            tokens.append({'element':_str})
-        elif state == 'attribute_value' and _str:
-            _dict['attr_value'] = _str
-            tokens.append(_dict)
-        elif state == 'attribute_value_quoted':
-            raise FirewallError(INVALID_RULE, 'missing quote')
 
+        for r in functions.splitArgs(rule_str):
+            if "=" in r:
+                attr = r.split('=')
+                if len(attr) != 2 or not attr[0] or not attr[1]:
+                    raise FirewallError(INVALID_RULE, 'internal error in _lexer(): %s' % r)
+                tokens.append({'attr_name':attr[0], 'attr_value':attr[1]})
+            else:
+                tokens.append({'element':r})
         tokens.append({'element':'EOL'})
-        # sanity check: each token has to be either element or attribute, not both
-        for d in tokens:
-            if (d.get('element') and d.get('attr_name')) or \
-               (d.get('element') and d.get('attr_value')) or \
-               (d.get('attr_name') and not d.get('attr_value')) or \
-               (d.get('attr_value') and not d.get('attr_name')) or not d:
-                raise FirewallError(INVALID_RULE, 'internal error in _lexer()')
 
         return tokens
 
@@ -250,7 +220,6 @@ class Rich_Rule(object):
         self.audit = None
         self.action = None
 
-        #tokens = rule_str.split()
         tokens = self._lexer(rule_str)
         if tokens and tokens[0].get('element')  == 'EOL':
             raise FirewallError(INVALID_RULE, 'empty rule')
@@ -280,30 +249,39 @@ class Rich_Rule(object):
                         raise FirewallError(INVALID_RULE, "more than one 'destination' element")
                     elif element in ['protocol', 'service', 'port', 'icmp-block', \
                                   'masquerade', 'forward-port'] and self.element:
-                        raise FirewallError(INVALID_RULE, "more than one element")
+                        raise FirewallError(INVALID_RULE, "more than one element. There cannot be both '%s' and '%s' in one rule." % (element, self.element))
                     elif element == 'log' and self.log:
                         raise FirewallError(INVALID_RULE, "more than one 'log' element")
                     elif element == 'audit' and self.audit:
                         raise FirewallError(INVALID_RULE, "more than one 'audit' element")
                     elif element in ['accept', 'drop', 'reject'] and self.action:
-                        raise FirewallError(INVALID_RULE, "more than one 'action' element")
+                        raise FirewallError(INVALID_RULE, "more than one 'action' element. There cannot be both '%s' and '%s' in one rule." % (element, self.action))
                 else:
                     raise FirewallError(INVALID_RULE, "unknown element %s" % element)
 
             in_element = in_elements[len(in_elements)-1] if len(in_elements) > 0 else ''
 
             if in_element == '':
-                if 'rule' not in element:
-                    raise FirewallError(INVALID_RULE, "'%s' outside of rule" % element)
+                if not element and attr_name:
+                    if attr_name == 'family':
+                        raise FirewallError(INVALID_RULE, "'family' outside of rule. Use 'rule family=...'.")
+                    else:
+                        raise FirewallError(INVALID_RULE, "'%s' outside of any element. Use 'rule <element> %s= ...'." % (attr_name, attr_name))
+                elif 'rule' not in element:
+                    raise FirewallError(INVALID_RULE, "'%s' outside of rule. Use 'rule ... %s ...'." % (element, element))
                 else:
                     in_elements.append('rule') # push into stack
             elif in_element == 'rule':
                 if attr_name == 'family':
                     if attr_value not in ['ipv4', 'ipv6']:
-                        raise FirewallError(INVALID_RULE, "bad 'family' attribute")
+                        raise FirewallError(INVALID_RULE, "'family' attribute cannot have '%s' value. Use 'ipv4' or 'ipv6' instead." % attr_value)
                     self.family = attr_value
                 elif attr_name:
-                    raise FirewallError(INVALID_RULE, "attribute '%s' outside of element" % attr_name)
+                    if attr_name == 'protocol':
+                        err_msg = "wrong 'protocol' usage. Use either 'rule protocol value=...' or  'rule [forward-]port protocol=...'."
+                    else:
+                        err_msg = "attribute '%s' outside of any element. Use 'rule <element> %s= ...'." % (attr_name, attr_name)
+                    raise FirewallError(INVALID_RULE, err_msg)
                 else:
                     in_elements.append(element) # push into stack
             elif in_element == 'source':
@@ -501,12 +479,12 @@ class Rich_Rule(object):
                 raise FirewallError(INVALID_PORT, self.element.port)
             if not self.element.protocol in [ "tcp", "udp" ]:
                 raise FirewallError(INVALID_PROTOCOL, self.element.protocol)
-            if self.element.to_port == None and self.element.to_address == None:
+            if self.element.to_port == "" and self.element.to_address == "":
                 raise FirewallError(INVALID_PORT, self.element.to_port)
-            if self.element.to_port != None and \
+            if self.element.to_port != "" and \
                     not functions.check_port(self.element.to_port):
                 raise FirewallError(INVALID_PORT, self.element.to_port)
-            if self.element.to_address != None and \
+            if self.element.to_address != "" and \
                     not functions.check_single_address(self.family,
                                                        self.element.to_address):
                 raise FirewallError(INVALID_ADDR, self.element.to_address)
@@ -520,8 +498,9 @@ class Rich_Rule(object):
 
         # log
         if self.log != None:
-            if self.log.level not in [ "emerg", "alert", "crit", "err",
-                                       "warn", "notice", "info", "debug" ]:
+            if self.log.level and \
+               self.log.level not in [ "emerg", "alert", "crit", "error",
+                                       "warning", "notice", "info", "debug" ]:
                 raise FirewallError(INVALID_LOG_LEVEL, self.log.level)
 
             if self.log.limit != None:
