@@ -27,18 +27,18 @@ PANIC_MODE          =   15
 ZONE_ALREADY_SET    =   16
 UNKNOWN_INTERFACE   =   17
 ZONE_CONFLICT       =   18
-IMMUTABLE           =   19
-BUILTIN_CHAIN       =   20
-EBTABLES_NO_REJECT  =   21
-NOT_OVERLOADABLE    =   22
-NO_DEFAULTS         =   23
-BUILTIN_ZONE        =   24
-BUILTIN_SERVICE     =   25
-BUILTIN_ICMPTYPE    =   26
-NAME_CONFLICT       =   27
-NAME_MISMATCH       =   28
-PARSE_ERROR         =   29
-ACCESS_DENIED       =   30
+BUILTIN_CHAIN       =   19
+EBTABLES_NO_REJECT  =   20
+NOT_OVERLOADABLE    =   21
+NO_DEFAULTS         =   22
+BUILTIN_ZONE        =   23
+BUILTIN_SERVICE     =   24
+BUILTIN_ICMPTYPE    =   25
+NAME_CONFLICT       =   26
+NAME_MISMATCH       =   27
+PARSE_ERROR         =   28
+ACCESS_DENIED       =   29
+UNKNOWN_SOURCE      =   30
 
 INVALID_ACTION      =  100
 INVALID_SERVICE     =  101
@@ -64,6 +64,10 @@ INVALID_SETTING     =  120
 INVALID_DESTINATION =  121
 INVALID_RULE        =  122
 INVALID_LIMIT       =  123
+INVALID_FAMILY      =  124
+INVALID_LOG_LEVEL   =  125
+INVALID_AUDIT_TYPE  =  126
+INVALID_MARK        =  127
 
 MISSING_TABLE       =  200
 MISSING_CHAIN       =  201
@@ -72,6 +76,7 @@ MISSING_PROTOCOL    =  203
 MISSING_ADDR        =  204
 MISSING_NAME        =  205
 MISSING_SETTING     =  206
+MISSING_FAMILY      =  207
 
 NOT_RUNNING         =  252
 NOT_AUTHORIZED      =  253
@@ -101,5 +106,12 @@ class FirewallError(Exception):
             ecode = msg[:idx]
         else:
             ecode = msg
-        return FirewallError.codes[ecode]
+
+        try:
+            code = FirewallError.codes[ecode]
+        except KeyError:
+            code = UNKNOWN_ERROR
+
+        return code
+
     get_code = staticmethod(get_code)
