@@ -40,6 +40,7 @@ from firewall.core.io.zone import Zone
 from firewall.core.io.service import Service
 from firewall.core.io.icmptype import IcmpType
 from firewall.core.io.lockdown_whitelist import LockdownWhitelist
+from firewall.core.io.direct import Direct
 from firewall.dbus_utils import dbus_to_python, \
     command_of_sender, context_of_sender, uid_of_sender, user_of_uid
 from firewall.errors import *
@@ -71,6 +72,8 @@ class FirewallDConfig(slip.dbus.service.Object):
         self.watcher.add_watch_dir(ETC_FIREWALLD_SERVICES)
         self.watcher.add_watch_dir(FIREWALLD_ZONES)
         self.watcher.add_watch_dir(ETC_FIREWALLD_ZONES)
+        self.watcher.add_watch_file(LOCKDOWN_WHITELIST)
+        self.watcher.add_watch_file(FIREWALLD_DIRECT)
 
     @handle_exceptions
     def _init_vars(self):
@@ -123,7 +126,7 @@ class FirewallDConfig(slip.dbus.service.Object):
             elif what == "update":
                 self._updateIcmpType(obj)
 
-        if name.startswith(FIREWALLD_SERVICES) or \
+        elif name.startswith(FIREWALLD_SERVICES) or \
                 name.startswith(ETC_FIREWALLD_SERVICES):
             (what, obj) = self.config.update_service_from_path(name)
             if what == "new":
@@ -133,7 +136,7 @@ class FirewallDConfig(slip.dbus.service.Object):
             elif what == "update":
                 self._updateService(obj)
 
-        if name.startswith(FIREWALLD_ZONES) or \
+        elif name.startswith(FIREWALLD_ZONES) or \
                 name.startswith(ETC_FIREWALLD_ZONES):
             (what, obj) = self.config.update_zone_from_path(name)
             if what == "new":
@@ -143,6 +146,15 @@ class FirewallDConfig(slip.dbus.service.Object):
             elif what == "update":
                 self._updateZone(obj)
 
+        elif name == LOCKDOWN_WHITELIST:
+            self.config.update_lockdown_whitelist()
+            self.LockdownWhitelistUpdated()
+
+        elif name == FIREWALLD_DIRECT:
+            self.config.update_direct()
+            self.Updated()
+
+
     @handle_exceptions
     def _addIcmpType(self, obj):
         # TODO: check for idx overflow
@@ -385,23 +397,23 @@ class FirewallDConfig(slip.dbus.service.Object):
                          out_signature=LockdownWhitelist.DBUS_SIGNATURE)
     @dbus_handle_exceptions
     def getLockdownWhitelist(self, sender=None):
-        log.debug1("config.getLockdownWhitelist()")
+        log.debug1("config.policies.getLockdownWhitelist()")
         return self.config.get_policies().lockdown_whitelist.export_config()
 
     @dbus_service_method(DBUS_INTERFACE_CONFIG_POLICIES, 
                          in_signature=LockdownWhitelist.DBUS_SIGNATURE)
     @dbus_handle_exceptions
     def setLockdownWhitelist(self, settings, sender=None):
-        log.debug1("config.setLockdownWhitelistSettings(...)")
+        log.debug1("config.policies.setLockdownWhitelistSettings(...)")
         settings = dbus_to_python(settings)
         self.config.get_policies().lockdown_whitelist.import_config(settings)
         self.config.get_policies().lockdown_whitelist.write()
         self.LockdownWhitelistUpdated()
 
-    @dbus.service.signal(DBUS_INTERFACE_CONFIG_POLICIES,)
+    @dbus.service.signal(DBUS_INTERFACE_CONFIG_POLICIES)
     @dbus_handle_exceptions
     def LockdownWhitelistUpdated(self):
-        log.debug1("config.LockdownWhitelistUpdated()")
+        log.debug1("config.policies.LockdownWhitelistUpdated()")
 
     # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 
@@ -569,3 +581,30 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def ZoneAdded(self, zone):
         log.debug1("config.ZoneAdded('%s')" % (zone))
+
+    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
+    # DIRECT
+
+    @dbus_service_method(DBUS_INTERFACE_CONFIG_DIRECT,
+                         out_signature=Direct.DBUS_SIGNATURE)
+    @dbus_handle_exceptions
+    def getSettings(self, sender=None):
+        # returns list ipv, table, list of chains
+        log.debug1("config.direct.getSettings()")
+        return self.config.get_direct().export_config()
+
+    @dbus_service_method(DBUS_INTERFACE_CONFIG_DIRECT,
+                         in_signature=Direct.DBUS_SIGNATURE)
+    @dbus_handle_exceptions
+    def update(self, settings, sender=None):
+        # returns list ipv, table, list of chains
+        log.debug1("config.direct.update()")
+        settings = dbus_to_python(settings)
+        self.config.get_direct().import_config(settings)
+        self.config.get_direct().write()
+        self.Updated()
+
+    @dbus.service.signal(DBUS_INTERFACE_CONFIG_DIRECT)
+    @dbus_handle_exceptions
+    def Updated(self):
+        log.debug1("config.direct.Updated()")
